import logging

from ._consts import RH_SAML_URL, AwsRegion
from ._core import (
    assume_role_with_saml,
    get_aws_accounts,
    get_saml_auth,
    is_kerberos_ticket_valid,
    select_aws_account,
)
from ._exceptions import NoAwsAccountError, NoKerberosTicketError
from ._models import AwsCredentials

logger = logging.getLogger(__name__)


def get_aws_credentials(
    account_name: str,
    saml_url: str = RH_SAML_URL,
    session_timeout_seconds: int = 900,
    region: str = AwsRegion.US_EAST_1,
) -> AwsCredentials:
    """Get AWS credentials for the given account name non-interactively."""
    if not is_kerberos_ticket_valid():
        raise NoKerberosTicketError
    aws_url, saml_token = get_saml_auth(saml_url)
    aws_accounts = get_aws_accounts(
        aws_url, saml_token, session_timeout_seconds, region
    )
    if not (account := select_aws_account(aws_accounts, account_name)):
        raise NoAwsAccountError(account_name)
    return assume_role_with_saml(account, saml_token)



import base64
import logging
import os
import re
import subprocess
import sys
import tempfile
import xml.etree.ElementTree as ET  # noqa: S405

import boto3
import botocore
import requests
from iterfzf import iterfzf
from pyquery import PyQuery as pq  # noqa: N813
from requests_kerberos import OPTIONAL, HTTPKerberosAuth

from ._models import AwsAccount, AwsCredentials
from ._utils import run

logger = logging.getLogger(__name__)


def is_kerberos_ticket_valid() -> bool:
    """Test for a valid kerberos ticket."""
    try:
        run(["klist", "-s"], check=True)
        return True
    except subprocess.CalledProcessError:
        return False


def kinit(kerberos_keytab: str | None, kerberos_principal: str) -> None:
    """Acquire a kerberos ticket."""
    cmd = ["kinit"]
    with tempfile.NamedTemporaryFile() as keytab_file:
        if kerberos_keytab:
            # decode base64 keytab file and write it to a temporary file
            keytab_file.write(base64.b64decode(kerberos_keytab))
            keytab_file.flush()
            cmd += ["-kt", keytab_file.name]
        run([*cmd, kerberos_principal], check=True, capture_output=False)


def get_saml_auth(url: str) -> tuple[str, str]:
    """Get the SAML token and AWS authentification URL."""
    with requests.Session() as session:
        session.auth = HTTPKerberosAuth(mutual_authentication=OPTIONAL)
        r = session.get(url)
        r.raise_for_status()
        p = pq(r.text).xhtml_to_html()
        form = p("form")
        aws_url = form.attr("action")
        saml_token = form("input:hidden").attr("value")
    return aws_url, saml_token


def get_single_account_from_saml(saml_token: str) -> AwsAccount | None:
    """Return an AWS account from the SAML token if exactly one role is found."""
    saml_response_xml = base64.b64decode(saml_token).decode("utf-8")
    root = ET.fromstring(saml_response_xml)  # noqa: S314
    namespaces = {
        "samlp": "urn:oasis:names:tc:SAML:2.0:protocol",
        "saml": "urn:oasis:names:tc:SAML:2.0:assertion",
    }
    if (
        role_attribute := root.find(
            ".//saml:Attribute[@Name='https://aws.amazon.com/SAML/Attributes/Role']",
            namespaces,
        )
    ) is None:
        msg = "No role attribute found in SAML response"
        raise ValueError(msg)

    role_values = role_attribute.findall("./saml:AttributeValue", namespaces)

    if len(role_values) == 1:
        # Exactly one role found
        if (role_value_element := role_values[0]) is None:
            msg = "No role value found in SAML response"
            raise ValueError(msg)
        if not role_value_element.text:
            msg = "Empty role value found in SAML response"
            raise ValueError(msg)

        role_arn = role_value_element.text.split(",")[0]

        arn_parts = role_arn.split(":")
        return AwsAccount(
            # the SAML response does not provide a friendly name, so we use the account ID and role name
            name=arn_parts[4],
            uid=arn_parts[4],
            role_name=arn_parts[5].split("/")[1],
            role_arn=role_arn,
        )
    return None


def get_aws_accounts(
    aws_url: str, saml_token: str, saml_token_duration_seconds: int, region: str
) -> list[AwsAccount]:
    """Get all AWS accounts accessible to the user."""
    # The AWS SAML login page redirects directly to the account console when only one account is found.
    # Unfortunately, the SAML token does not contain the account names.
    # So we stick with the AWS SAML login html parsing if the user has multiple accounts.
    if aws_account := get_single_account_from_saml(saml_token):
        aws_account.session_timeout_seconds = saml_token_duration_seconds
        aws_account.region = region
        return [aws_account]

    r = requests.post(aws_url, data={"SAMLResponse": saml_token}, timeout=10)
    r.raise_for_status()

    p = pq(r.text).xhtml_to_html()
    accounts = p("div.saml-account")
    if not accounts:
        errormsg = f"No AWS accounts found: {r.text}"
        logger.error(errormsg)
        raise ValueError(errormsg)

    aws_accounts = []
    for account in accounts.items():
        name = account.find(".saml-account-name").text()
        if not name:
            continue
        # "Account: foobar (123456789)" or just with the ID "Account: 123456789"
        name = re.split(r"\s+", name)[1]
        role_labels = account.find(".saml-role").find("label")
        for role_label in role_labels.items():
            # arn:aws:iam::123456789:role/123456789-role-name
            role_arn = role_label.attr("for")
            aws_accounts.append(
                AwsAccount(
                    name=name,
                    uid=role_arn.split(":")[4],
                    role_name=role_label.text(),
                    role_arn=role_arn,
                    session_timeout_seconds=saml_token_duration_seconds,
                    region=region,
                )
            )
    return aws_accounts


def select_aws_account(
    aws_accounts: list[AwsAccount], account_name: str, role: str | None = None
) -> AwsAccount | None:
    """Select an AWS account from the list of available accounts."""
    return next(
        (
            a
            for a in aws_accounts
            if a.name == account_name and (not role or a.role_name == role)
        ),
        None,
    )


def get_aws_account(
    aws_accounts: list[AwsAccount], account_name: str | None, role: str | None = None
) -> AwsAccount | None:
    """Select and return an AWS account from the list of available accounts."""
    if len(aws_accounts) == 1:
        return aws_accounts[0]

    if not account_name:
        items = [f"{acc.name:<40} {acc.role_name}" for acc in aws_accounts]
        selected_item = iterfzf(
            items,
            exact=True,
            __extra__=[f"--header={'Account':<40} Role"],
        )
        if not selected_item:
            sys.exit(0)
        account_name, role = re.split(r"\s+", selected_item, maxsplit=1)

    if account_name == ".":
        # account name can be passed as a dot to open the console for the previously selected account
        account_name = os.environ.get("AWS_ACCOUNT_NAME")

    assert account_name  # make mypy happy
    return select_aws_account(aws_accounts, account_name, role)


def assume_role_with_saml(account: AwsAccount, saml_token: str) -> AwsCredentials:
    """Assume a role with SAML token."""
    sts = boto3.client(
        "sts",
        config=botocore.config.Config(signature_version=botocore.UNSIGNED),
        region_name=account.region,
    )
    response = sts.assume_role_with_saml(
        RoleArn=account.role_arn,
        PrincipalArn=account.principle_arn,
        SAMLAssertion=saml_token,
        DurationSeconds=account.session_timeout_seconds,
    )
    return AwsCredentials(
        access_key=response["Credentials"]["AccessKeyId"],
        secret_key=response["Credentials"]["SecretAccessKey"],
        session_token=response["Credentials"]["SessionToken"],
        expiration=response["Credentials"]["Expiration"],
        session_timeout_seconds=account.session_timeout_seconds,
        region=account.region,
    )


def assume_role(account: AwsAccount, credentials: AwsCredentials) -> AwsCredentials:
    """Assume a role with the given credentials."""
    sts = boto3.client(
        "sts",
        aws_access_key_id=credentials.access_key,
        aws_secret_access_key=credentials.secret_key,
        aws_session_token=credentials.session_token,
        region_name=account.region,
    )
    response = sts.assume_role(
        RoleArn=account.role_arn,
        RoleSessionName=account.role_name,
    )
    return AwsCredentials(
        access_key=response["Credentials"]["AccessKeyId"],
        secret_key=response["Credentials"]["SecretAccessKey"],
        session_token=response["Credentials"]["SessionToken"],
        expiration=response["Credentials"]["Expiration"],
        session_timeout_seconds=account.session_timeout_seconds,
        region=account.region,
    )



import json
import logging
import os
import shlex
import sys
import urllib
from collections.abc import Generator
from datetime import UTC
from datetime import datetime as dt
from enum import StrEnum
from importlib.metadata import version
from pathlib import Path
from textwrap import dedent
from typing import Annotated

import humanize
import requests
import typer
from rich import print as rich_print
from rich.progress import Progress, SpinnerColumn, TextColumn
from tzlocal import get_localzone

from ._consts import RH_SAML_URL, AwsConsoleService, AwsRegion
from ._core import (
    assume_role,
    assume_role_with_saml,
    get_aws_account,
    get_aws_accounts,
    get_saml_auth,
    is_kerberos_ticket_valid,
    kinit,
)
from ._models import AwsAccount, AwsCredentials
from ._utils import blend_text, bye, enable_requests_logging, run

app = typer.Typer(rich_markup_mode="rich")
BANNER = r"""
         __                                                         __      __            _
   _____/ /_        ____ __      _______      _________ _____ ___  / /     / /___  ____ _(_)___
  / ___/ __ \______/ __ `/ | /| / / ___/_____/ ___/ __ `/ __ `__ \/ /_____/ / __ \/ __ `/ / __ \
 / /  / / / /_____/ /_/ /| |/ |/ (__  )_____(__  ) /_/ / / / / / / /_____/ / /_/ / /_/ / / / / /
/_/  /_/ /_/      \__,_/ |__/|__/____/     /____/\__,_/_/ /_/ /_/_/     /_/\____/\__, /_/_/ /_/
                                                                                /____/
"""
APP_NAME = "rh-aws-saml-login"
APP_DIR = Path(typer.get_app_dir(APP_NAME))
APP_DIR.mkdir(exist_ok=True)
ACCOUNT_CACHE = APP_DIR / "account_cache.json"

SCRIPT_START_TIME = dt.now(UTC)

logger = logging.getLogger(__name__)


class OutputFormat(StrEnum):
    """Supported output formats"""

    JSON = "json"
    ENV = "env"


def get_export_environment_variables(
    account: AwsAccount,
    credentials: AwsCredentials,
    region: str,
) -> dict:
    """Get the environment variables to export AWS credentials."""
    return {
        "AWS_ACCOUNT_NAME": account.name,
        "AWS_ACCOUNT_UID": account.uid,
        "AWS_ROLE_NAME": account.role_name,
        "AWS_ROLE_ARN": account.role_arn,
        "AWS_ACCESS_KEY_ID": credentials.access_key,
        "AWS_SECRET_ACCESS_KEY": credentials.secret_key,
        "AWS_SESSION_TOKEN": credentials.session_token,
        "AWS_REGION": region,
    }


def open_aws_shell(
    account: AwsAccount,
    credentials: AwsCredentials,
    region: str,
    command: list[str] | str | None = None,
    *,
    quiet: bool = False,
) -> None:
    if not quiet and not command:
        rich_print(
            dedent(f"""
            Spawning a new shell. Use exit or CTRL+d to leave it!

            :nerd_face: {account.name}
            :rocket: {account.role_name}
            :hourglass: {humanize.naturaltime(credentials.expiration, when=SCRIPT_START_TIME)} ({credentials.expiration.astimezone(tz=get_localzone())})
        """)
        )
    if not command:
        command = os.environ.get("SHELL", "/bin/bash")
    run(
        command,
        check=False,
        capture_output=False,
        env=get_export_environment_variables(account, credentials, region),
    )


def open_aws_console(
    open_command: str, credentials: AwsCredentials, console_service: str | None = None
) -> None:
    """Open the AWS console in a browser.

    See https://docs.aws.amazon.com/IAM/latest/UserGuide/id_roles_providers_enable-console-custom-url.html
    """
    aws_federated_signin_endpoint = "https://signin.aws.amazon.com/federation"
    # Get a sign-in token from the AWS sign-in federation endpoint.
    response = requests.get(
        aws_federated_signin_endpoint,
        params={
            "Action": "getSigninToken",
            "SessionDuration": str(credentials.session_timeout_seconds),
            "Session": json.dumps({
                "sessionId": credentials.access_key,
                "sessionKey": credentials.secret_key,
                "sessionToken": credentials.session_token,
            }),
        },
        timeout=10,
    )
    if response.status_code == requests.codes.BAD_REQUEST:
        logger.error(
            "Failed to get a sign-in token. Try lowering the session timeout value via --session-timeout."
        )
        sys.exit(1)

    signin_token = json.loads(response.text)
    # Make a federated URL that can be used to sign into the AWS Management Console.
    query_string = urllib.parse.urlencode({
        "Action": "login",
        "Issuer": "redhat.com",
        "Destination": f"https://{credentials.region}.console.aws.amazon.com/{console_service or ''}",
        "SigninToken": signin_token["SigninToken"],
    })
    federated_url = f"{aws_federated_signin_endpoint}?{query_string}"
    run([*shlex.split(open_command), federated_url], check=False, capture_output=False)


def display_credentials(
    account: AwsAccount, credentials: AwsCredentials, region: str, output: OutputFormat
) -> None:
    """Display AWS credentials in the specified output format."""
    env_vars = get_export_environment_variables(account, credentials, region)
    match output:
        case OutputFormat.JSON:
            print(json.dumps(env_vars, indent=2))  # noqa: T201
        case OutputFormat.ENV:
            for key, value in env_vars.items():
                print(f"{key}={value}")  # noqa: T201


def write_accounts_cache(accounts: list[str]) -> None:
    """Write accounts cache to disk."""
    json.dump(accounts, ACCOUNT_CACHE.open("w"))


def read_accounts_cache() -> list[str]:
    """Read accounts cache from disk."""
    if ACCOUNT_CACHE.exists():
        return json.load(ACCOUNT_CACHE.open())
    return []


def complete_account(ctx: typer.Context, incomplete: str) -> Generator[str, None, None]:  # noqa: ARG001
    for name in read_accounts_cache():
        if name.startswith(incomplete):
            yield name


def version_callback(*, value: bool) -> None:
    if value:
        rich_print(f"Version: {version(APP_NAME)}")
        raise typer.Exit


@app.command(epilog="Made with [red]:heart:[/] by [blue]https://github.com/app-sre[/]")
def cli(  # noqa: PLR0917
    account_name: Annotated[
        str | None,
        typer.Argument(
            help="AWS account name. Supports: account name (e.g., 'my-account'), account/role format (e.g., 'my-account/PowerUserAccess'), or '.' for $AWS_ACCOUNT_NAME environment variable.",
            autocompletion=complete_account,
        ),
    ] = None,
    command: Annotated[list[str] | None, typer.Argument(help="Command to run")] = None,
    region: Annotated[
        AwsRegion,
        typer.Option(
            help="AWS region",
            envvar="RH_AWS_REGION",
        ),
    ] = AwsRegion.US_EAST_1,
    saml_url: Annotated[
        str,
        typer.Option(
            help="SAML URL",
        ),
    ] = RH_SAML_URL,
    session_timeout: Annotated[
        int,
        typer.Option(
            help="Session timeout in minutes. Default: 60 minutes. Max value depends on the AWS IAM role.",
            envvar="RH_AWS_SESSION_TIMEOUT",
        ),
    ] = 60,
    open_command: Annotated[
        str,
        typer.Option(
            help="Command to open the browser (e.g. 'xdg-open' on Linux)",
            envvar="RH_AWS_SAML_LOGIN_OPEN_COMMAND",
        ),
    ] = "open",
    console_service: Annotated[
        AwsConsoleService | None,
        typer.Option(
            help="Directly open this AWS console service",
            envvar="RH_AWS_CONSOLE_SERVICE",
        ),
    ] = None,
    assume_uid: Annotated[
        str | None,
        typer.Option(
            help="Define the target AWS account UID to assume",
        ),
    ] = None,
    assume_role: Annotated[
        str,
        typer.Option(
            help="Define the role name to assume",
        ),
    ] = "role/OrganizationAccountAccessRole",
    *,
    debug: Annotated[bool, typer.Option(help="Enable debug mode")] = False,
    console: Annotated[
        bool,
        typer.Option(help="Open the AWS console in browser instead of a local shell"),
    ] = False,
    kerberos_keytab: Annotated[
        str | None,
        typer.Option(
            help="Base64 encoded Kerberos keytab file. If set, it will be used to acquire a Kerberos ticket.",
            envvar="RH_KERBEROS_KEYTAB",
        ),
    ] = None,
    kerberos_principal: Annotated[
        str,
        typer.Option(
            help="Kerberos principal to use for acquiring a Kerberos ticket.",
            envvar="RH_KERBEROS_PRINCIPAL",
        ),
    ] = "",
    quiet: Annotated[
        bool,
        typer.Option(
            help="Do not display any logging output or other messages!",
            envvar="RH_QUIET",
        ),
    ] = False,
    output: Annotated[
        OutputFormat | None,
        typer.Option(
            help="Instead of opening the AWS console or shell, output the credentials in the specified format. Also implies --quiet.",
            envvar="RH_OUTPUT",
            case_sensitive=False,
        ),
    ] = None,
    display_banner: Annotated[
        bool,
        typer.Option(
            help="Display a shiny banner",
            envvar="RH_DISPLAY_BANNER",
        ),
    ] = True,
    version: Annotated[  # noqa: ARG001
        bool | None, typer.Option("--version", callback=version_callback)
    ] = None,
) -> None:
    """Login to AWS using SAML."""
    log_level = logging.INFO
    if output:
        quiet = True
    if debug:
        log_level = logging.DEBUG
    if quiet:
        log_level = logging.ERROR
    logging.basicConfig(level=log_level, format="%(message)s")

    if display_banner and not quiet:
        rich_print(blend_text(BANNER, (32, 32, 255), (255, 32, 255)))
    if debug:
        enable_requests_logging()

    role = None
    if account_name and "/" in account_name:
        # when account_name contains a '/', split it into account_name and role
        account_name, role = account_name.split("/", 1)

    accounts = _main(
        account_name=account_name,
        role=role,
        region=region,
        console=console,
        saml_url=saml_url,
        session_timeout_seconds=session_timeout * 60,
        command=command,
        open_command=open_command,
        console_service=console_service,
        assume_uid=assume_uid,
        assume_role_name=assume_role,
        kerberos_keytab=kerberos_keytab,
        kerberos_principal=kerberos_principal,
        output=output,
        quiet=quiet,
    )
    write_accounts_cache(accounts)


def _main(  # noqa: PLR0917
    account_name: str | None,
    role: str | None,
    region: str,
    saml_url: str,
    session_timeout_seconds: int,
    command: list[str] | None,
    open_command: str,
    console_service: str | None,
    assume_uid: str | None,
    assume_role_name: str,
    kerberos_keytab: str | None = None,
    kerberos_principal: str = "",
    output: OutputFormat | None = None,
    *,
    console: bool,
    quiet: bool,
) -> list[str]:
    with Progress(
        SpinnerColumn(finished_text="✅"),
        TextColumn("[progress.description]{task.description}"),
        disable=quiet,
    ) as progress:
        task = progress.add_task(
            description="Test for a valid Kerberos ticket ...", total=1
        )
        if not is_kerberos_ticket_valid():
            progress.stop()
            logger.info("No valid Kerberos ticket found. Acquiring one ...")
            kinit(kerberos_keytab, kerberos_principal)
            progress.start()
        progress.update(task, completed=1)

        task = progress.add_task(description="Getting SAML token ...", total=1)
        aws_url, saml_token = get_saml_auth(saml_url)
        progress.update(task, completed=1)

        task = progress.add_task(description="Getting AWS accounts ...", total=1)
        aws_accounts = get_aws_accounts(
            aws_url, saml_token, session_timeout_seconds, region
        )

        progress.stop()
        if not (account := get_aws_account(aws_accounts, account_name, role)):
            if role:
                logger.error("Account with role not found: %s/%s", account_name, role)
            else:
                logger.error("Account not found: %s", account_name)
            sys.exit(1)
        progress.start()
        progress.update(task, completed=1)

        task = progress.add_task(
            description="Getting temporary AWS credentials ...", total=1
        )
        credentials = assume_role_with_saml(account, saml_token)
        progress.update(task, completed=1)

        if assume_uid:
            account = AwsAccount(
                name=assume_uid,
                uid=assume_uid,
                role_name=account.name,
                role_arn=f"arn:aws:iam::{assume_uid}:{assume_role_name}",
                session_timeout_seconds=session_timeout_seconds,
                region=region,
            )
            task = progress.add_task(description="Assume role ...", total=1)
            credentials = assume_role(account, credentials)
            progress.update(task, completed=1)

    if output:
        display_credentials(account, credentials, region, output)
    elif console:
        open_aws_console(open_command, credentials, console_service)
    else:
        open_aws_shell(account, credentials, region, command, quiet=quiet)
    if not quiet:
        bye()
    return [acc.name for acc in aws_accounts]



"""Expose the public API of the package."""

from ._api import get_aws_credentials
from ._exceptions import NoAwsAccountError, NoKerberosTicketError
from ._models import AwsCredentials

__all__ = [
    "AwsCredentials",
    "NoAwsAccountError",
    "NoKerberosTicketError",
    "get_aws_credentials",
]



from enum import StrEnum

RH_SAML_URL = (
    "https://auth.redhat.com/auth/realms/EmployeeIDP/protocol/saml/clients/itaws"
)


# boto3.session.Session().get_available_regions("rds")  # noqa: ERA001
class AwsRegion(StrEnum):
    AF_SOUTH_1 = "af-south-1"
    AP_EAST_1 = "ap-east-1"
    AP_NORTHEAST_1 = "ap-northeast-1"
    AP_NORTHEAST_2 = "ap-northeast-2"
    AP_NORTHEAST_3 = "ap-northeast-3"
    AP_SOUTH_1 = "ap-south-1"
    AP_SOUTH_2 = "ap-south-2"
    AP_SOUTHEAST_1 = "ap-southeast-1"
    AP_SOUTHEAST_2 = "ap-southeast-2"
    AP_SOUTHEAST_3 = "ap-southeast-3"
    AP_SOUTHEAST_4 = "ap-southeast-4"
    AP_SOUTHEAST_5 = "ap-southeast-5"
    CA_CENTRAL_1 = "ca-central-1"
    CA_WEST_1 = "ca-west-1"
    EU_CENTRAL_1 = "eu-central-1"
    EU_CENTRAL_2 = "eu-central-2"
    EU_NORTH_1 = "eu-north-1"
    EU_SOUTH_1 = "eu-south-1"
    EU_SOUTH_2 = "eu-south-2"
    EU_WEST_1 = "eu-west-1"
    EU_WEST_2 = "eu-west-2"
    EU_WEST_3 = "eu-west-3"
    IL_CENTRAL_1 = "il-central-1"
    ME_CENTRAL_1 = "me-central-1"
    ME_SOUTH_1 = "me-south-1"
    MX_CENTRAL_1 = "mx-central-1"
    SA_EAST_1 = "sa-east-1"
    US_EAST_1 = "us-east-1"
    US_EAST_2 = "us-east-2"
    US_WEST_1 = "us-west-1"
    US_WEST_2 = "us-west-2"


class AwsConsoleService(StrEnum):
    ACM = "acm"
    ACM_PCA = "acm-pca"
    AMPLIFY = "amplify"
    APIGATEWAY = "apigateway"
    APPFABRIC = "appfabric"
    APPFLOW = "appflow"
    APPMESH = "appmesh"
    APPRUNNER = "apprunner"
    APPSYNC = "appsync"
    APPTEST = "apptest"
    ARTIFACT = "artifact"
    ATHENA = "athena"
    AUDITMANAGER = "auditmanager"
    B2BI = "b2bi"
    BACKUP = "backup"
    BATCH = "batch"
    BEDROCK = "bedrock"
    BILLING = "billing"
    BILLINGCONDUCTOR = "billingconductor"
    BRAKET = "braket"
    CHATBOT = "chatbot"
    CLEANROOMS = "cleanrooms"
    CLOUD9 = "cloud9"
    CLOUDFORMATION = "cloudformation"
    CLOUDFRONT = "cloudfront"
    CLOUDHSM = "cloudhsm"
    CLOUDSEARCH = "cloudsearch"
    CLOUDTRAIL = "cloudtrail"
    CLOUDWATCH = "cloudwatch"
    CODEBUILD = "codebuild"
    CODECATALYST = "codecatalyst"
    CODECOMMIT = "codecommit"
    CODEDEPLOY = "codedeploy"
    CODEPIPELINE = "codepipeline"
    COMPREHEND = "comprehend"
    COMPREHENDMEDICAL = "comprehendmedical"
    COMPUTE_OPTIMIZER = "compute-optimizer"
    CONFIG = "config"
    CONNECT = "connect"
    CONTROLTOWER = "controltower"
    CUSTOMER_PROFILES = "customer-profiles"
    DATABREW = "databrew"
    DATAEXCHANGE = "dataexchange"
    DATASYNC = "datasync"
    DATAZONE = "datazone"
    DETECTIVE = "detective"
    DEVICEFARM = "devicefarm"
    DEVOPS_GURU = "devops-guru"
    DISCOVERY = "discovery"
    DOCDB = "docdb"
    DRS = "drs"
    DSQL = "dsql"
    DYNAMODB = "dynamodb"
    EC2 = "ec2"
    ECR = "ecr"
    ECS = "ecs"
    EFS = "efs"
    EKS = "eks"
    ELASTICACHE = "elasticache"
    ELASTICBEANSTALK = "elasticbeanstalk"
    ELASTICTRANSCODER = "elastictranscoder"
    EMR = "emr"
    ENTITYRESOLUTION = "entityresolution"
    EVENTS = "events"
    EVIDENTLY = "evidently"
    FINSPACE = "finspace"
    FIREHOSE = "firehose"
    FIS = "fis"
    FORECAST = "forecast"
    FRAUDDETECTOR = "frauddetector"
    FSX = "fsx"
    GAMELIFT = "gamelift"
    GLACIER = "glacier"
    GLOBALACCELERATOR = "globalaccelerator"
    GLUE = "glue"
    GRAFANA = "grafana"
    GREENGRASS = "greengrass"
    GROUNDSTATION = "groundstation"
    GUARDDUTY = "guardduty"
    HEALTH = "health"
    HEALTHLAKE = "healthlake"
    IAM = "iam"
    IMAGEBUILDER = "imagebuilder"
    IMPORTEXPORT = "importexport"
    INSPECTOR = "inspector"
    IOT = "iot"
    IOTANALYTICS = "iotanalytics"
    IOTEVENTS = "iotevents"
    IOTFLEETWISE = "iotfleetwise"
    IOTSITEWISE = "iotsitewise"
    IOTTWINMAKER = "iottwinmaker"
    IOTWIRELESS = "iotwireless"
    IVS = "ivs"
    KENDRA = "kendra"
    KEYSPACES = "keyspaces"
    KINESIS = "kinesis"
    KINESISANALYTICS = "kinesisanalytics"
    KINESISVIDEO = "kinesisvideo"
    KMS = "kms"
    LAKEFORMATION = "lakeformation"
    LAMBDA = "lambda"
    LICENSE_MANAGER = "license-manager"
    LOCATION = "location"
    LOOKOUTEQUIPMENT = "lookoutequipment"
    LOOKOUTMETRICS = "lookoutmetrics"
    LOOKOUTVISION = "lookoutvision"
    M2 = "m2"
    MACHINELEARNING = "machinelearning"
    MANAGEDBLOCKCHAIN = "managedblockchain"
    MEDIACONNECT = "mediaconnect"
    MEDIACONVERT = "mediaconvert"
    MEDIALIVE = "medialive"
    MEDIAPACKAGE = "mediapackage"
    MEDIASTORE = "mediastore"
    MEDIATAILOR = "mediatailor"
    MEDICAL_IMAGING = "medical-imaging"
    MEMORYDB = "memorydb"
    MGN = "mgn"
    MTURK = "mturk"
    MWAA = "mwaa"
    NEPTUNE = "neptune"
    NETWORKMANAGER = "networkmanager"
    NETWORKMONITOR = "networkmonitor"
    NOTIFICATIONS = "notifications"
    OMICS = "omics"
    OPSWORKS = "opsworks"
    ORGANIZATIONS = "organizations"
    OUTPOSTS = "outposts"
    PANORAMA = "panorama"
    PAYMENT_CRYPTOGRAPHY = "payment-cryptography"
    PCA_CONNECTOR_AD = "pca-connector-ad"
    PCA_CONNECTOR_SCEP = "pca-connector-scep"
    PCS = "pcs"
    PERSONALIZE = "personalize"
    PINPOINT = "pinpoint"
    POLLY = "polly"
    PROTON = "proton"
    QLDB = "qldb"
    RAM = "ram"
    RBIN = "rbin"
    RDS = "rds"
    REDSHIFT = "redshift"
    REKOGNITION = "rekognition"
    RESILIENCEHUB = "resiliencehub"
    RESOURCE_GROUPS = "resource-groups"
    ROBOMAKER = "robomaker"
    ROLESANYWHERE = "rolesanywhere"
    ROUTE53 = "route53"
    ROUTE53PROFILES = "route53profiles"
    ROUTE53RESOLVER = "route53resolver"
    S3 = "s3"
    SAGEMAKER = "sagemaker"
    SCHEDULER = "scheduler"
    SECRETSMANAGER = "secretsmanager"
    SECURITY_IR = "security-ir"
    SECURITYHUB = "securityhub"
    SECURITYLAKE = "securitylake"
    SERVERLESSREPO = "serverlessrepo"
    SERVICECATALOG = "servicecatalog"
    SES = "ses"
    SESV2 = "sesv2"
    SIGNER = "signer"
    SIMSPACEWEAVER = "simspaceweaver"
    SMS_VOICE = "sms-voice"
    SNS = "sns"
    SQS = "sqs"
    SSM = "ssm"
    STORAGEGATEWAY = "storagegateway"
    SUPPORT = "support"
    SYNTHETICS = "synthetics"
    TEXTRACT = "textract"
    TNB = "tnb"
    TRANSCRIBE = "transcribe"
    TRANSFER = "transfer"
    TRANSLATE = "translate"
    TRUSTEDADVISOR = "trustedadvisor"
    VERIFIEDPERMISSIONS = "verifiedpermissions"
    WAFV2 = "wafv2"
    WELLARCHITECTED = "wellarchitected"
    WORKMAIL = "workmail"
    WORKSPACES = "workspaces"
    WORKSPACES_THIN_CLIENT = "workspaces-thin-client"
    WORKSPACES_WEB = "workspaces-web"
    XRAY = "xray"



from dataclasses import dataclass
from datetime import datetime as dt


@dataclass
class AwsAccount:
    name: str
    uid: str
    role_name: str
    role_arn: str
    session_timeout_seconds: int = 3600
    region: str = "us-east-1"

    @property
    def principle_arn(self) -> str:
        return f"arn:aws:iam::{self.uid}:saml-provider/RedHatInternal"


@dataclass
class AwsCredentials:
    access_key: str
    secret_key: str
    session_token: str
    expiration: dt
    session_timeout_seconds: int
    region: str



class NoKerberosTicketError(RuntimeError):
    def __init__(self) -> None:
        super().__init__("No valid kerberos ticket found.")


class NoAwsAccountError(RuntimeError):
    def __init__(self, account_name: str) -> None:
        super().__init__(f"Account not found: {account_name}")
